//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-pdf-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Reflection;
using System.Collections.Generic;
using System.Diagnostics;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.Layers;
//
// This program creates an instance of the RoundRectangle class and uses it to generate a PDF file.
// It also saves the generated PDF as a multi-frame TIFF, and the first page as an SVG
// (other supported formats are JPEG, PNG, BMP and GIF).
// The packages needed to build and run this demo are available on NuGet.org.
//
// Command line switches:
// /y, -y -- assume all defaults, overwrite existing files.
// /q, -q -- quiet operation.
// /n, -n -- generate repeatable 'random' data.
// /f path, -f path -- use 'name' as base file path/name.
//
// Program exit codes:
// 0 - the program completed successfully.
// 1 - unexpected error.
// 2 - user abort.
//
var yes2all = false;
var quiet = false;
var norandom = false;
MemoryStream outMs = null;
TextWriter outTw = null;
try
{
    var fname = "RoundRectangle";
    for (int i = 0; i < args.Length; ++i)
    {
        switch (args[i].ToLowerInvariant())
        {
            case "/y":
            case "-y":
                yes2all = true;
                break;
            case "/q":
            case "-q":
                quiet = true;
                break;
            case "/n":
            case "-n":
                norandom = true;
                break;
            case "/f":
            case "-f":
                if (i + 1 < args.Length)
                {
                    if (Directory.Exists(args[++i]))
                        fname = Path.Combine(args[i], fname);
                    else
                        fname = args[i];
                }
                break;
        }
    }
    if (norandom)
        DsPdfWeb.Demos.Common.Util.NoRandom = true;
    if (!yes2all)
        quiet = false;
    if (quiet)
    {
        outMs = new MemoryStream();
        outTw = new StreamWriter(outMs);
        Console.SetOut(outTw);
    }

    var lk = Environment.GetEnvironmentVariable("DOCSOL_LKEY");
    if (!string.IsNullOrEmpty(lk))
        GcPdfDocument.SetLicenseKey(lk);

    var pdfName = Path.ChangeExtension(fname, ".pdf");
    var tiffName = Path.ChangeExtension(fname, ".tiff");
    var svgName = Path.ChangeExtension(fname, ".svg");

    if (!yes2all)
    {
        Console.WriteLine($"Press ENTER to create '{pdfName}' or enter an alternative file name:");
        var t = Console.ReadLine();
        if (!string.IsNullOrWhiteSpace(t))
            pdfName = t;
        pdfName = Path.GetFullPath(pdfName);
        if (File.Exists(pdfName))
        {
            Console.WriteLine($"'{pdfName}' exists.\nOVERWRITE (Y/n)?");
            var yesno = Console.ReadLine();
            if (yesno?.ToLower() == "n")
            {
                Console.WriteLine("Aborted.");
                return 2;
            }
        }
    }
    if (!quiet)
        Console.WriteLine($"Generating '{pdfName}'...");
    var sample = new DsPdfWeb.Demos.RoundRectangle();
    using (var fs = new FileStream(pdfName, FileMode.Create))
    {
        sample.CreatePDF(fs);
        if (!quiet)
            Console.WriteLine($"Created '{pdfName}' successfully.");

        if (!yes2all)
        {
            // Get the TIFF name:
            Console.WriteLine($"Press ENTER to create '{tiffName}' or enter an alternative file name:");
            var t = Console.ReadLine();
            if (!string.IsNullOrWhiteSpace(t))
                tiffName = t;
            tiffName = Path.GetFullPath(tiffName);
            if (File.Exists(tiffName))
            {
                Console.WriteLine($"'{tiffName}' exists.\nOVERWRITE (Y/n)?");
                var yesno = Console.ReadLine();
                if (yesno?.ToLower() == "n")
                {
                    Console.WriteLine("Aborted.");
                    return 2;
                }
            }
            // Get the SVG name:
            Console.WriteLine($"Press ENTER to create '{svgName}' or enter an alternative file name:");
            t = Console.ReadLine();
            if (!string.IsNullOrWhiteSpace(t))
                svgName = t;
            svgName = Path.GetFullPath(svgName);
            if (File.Exists(svgName))
            {
                Console.WriteLine($"'{svgName}' exists.\nOVERWRITE (Y/n)?");
                var yesno = Console.ReadLine();
                if (yesno?.ToLower() == "n")
                {
                    Console.WriteLine("Aborted.");
                    return 2;
                }
            }
        }
        // Save as TIFF:
        if (!quiet)
            Console.WriteLine($"Generating '{tiffName}'...");
        fs.Seek(0, SeekOrigin.Begin);
        var pdf = new GcPdfDocument();
        // NOTE: up to 5 pages will be loaded if DsPdf is not licensed:
        pdf.Load(fs);
        // Create default view state so that the default layers configuration is used in exports:
        var vs = new ViewState(pdf);
        pdf.SaveAsTiff(tiffName, vs);
        if (!quiet)
        {
            Console.WriteLine($"Created '{tiffName}' successfully.");
            Console.WriteLine($"Generating '{svgName}'...");
        }
        // Save as SVG:
        pdf.Pages[0].SaveAsSvg(svgName, vs);
        if (!quiet)
            Console.WriteLine($"Created '{svgName}' successfully.");

        // Document pages can be also saved to other supported image formats,
        // see GcPdfDocument.SaveAs<format> or Page.SaveAs<format> for details:
    }
    // Uncomment to open the generated PDF file:
    // var psi = new ProcessStartInfo() { FileName = pdfName, UseShellExecute = true };
    // Process.Start(psi);

    // Uncomment to open the generated TIFF file:
    // var psiTiff = new ProcessStartInfo() { FileName = tiffName, UseShellExecute = true };
    // Process.Start(psiTiff);

    // Uncomment to open the generated SVG file:
    // var psiSvg = new ProcessStartInfo() { FileName = svgName, UseShellExecute = true };
    // Process.Start(psiSvg);
}
catch (Exception ex)
{
    Console.Error.WriteLine($"Error: {ex.Message}");
    if (ex.InnerException != null)
        Console.Error.WriteLine($"InnerException: {ex.InnerException.Message}");
    return 1;
}
finally
{
    if (!yes2all)
    {
        Console.WriteLine("Press any key to exit...");
        Console.ReadKey();
    }
    if (quiet)
    {
        outTw.Dispose();
        outMs.Dispose();
    }
}
return 0;
